import { Timestamp } from "../../frontend/src/API.ts";
import { SchedulableTransaction } from "./transactions.ts";

export function emptyState(): State {
  return {
    instances: {},
    sessions: {},
    scheduledTransactions: [],
  };
}

// NOTE: This returns a shortened uuid to reduce log line width a bit.
export function uuid(): string {
  return crypto.randomUUID().substring(0, 13) + "X";
}

export interface State {
  instances: { [key: string]: InstanceState };
  sessions: { [key: string]: Session };
  scheduledTransactions: SchedulableTransaction[];
}

export interface InstanceState {
  uuid: string;
  teams: { [key: string]: Team };
  settings: Settings;
  authKey: string;
  buzzerEpoch: number;
  cooldown?: number;
  lastWinner?: string;
  buzzersEnabled: boolean;
  lastBuzz?: BuzzEvent;
  enableTimestamp?: Timestamp;
  buzzOrder: { teamUuid: string; timestamp: Timestamp }[];
}

export interface Session {
  uuid: string;
  instance: string;
  admin: boolean;
  deviceId?: string;
}

export interface BuzzEvent {
  teamUuid: string;
  timestamp: Timestamp;
}

export interface Device {
  uuid: string;
  name: string;
  lastSeenTimestamp: Timestamp;
}

export interface Team {
  uuid: string;
  name: string;
  altName?: string;
  excluded: boolean;
  buzzedDuringCooldown: boolean;
  devices: { [key: string]: Device };
  isLobby: boolean;
  score: number;
}

export interface ClientState {
  ownTeam: Team;
  deviceName: string;
}

export interface ServerState {
  teams: Team[];
  cooldown: number;
  lastWinner: string;
  buzzersEnabled: boolean;
  instanceId: string;
  enableTimestamp?: Timestamp;
  buzzOrder: { teamUuid: string; timestamp: Timestamp }[];
}

export interface Settings {
  timeout: number;
  rearmAfter: number;
  autoReenable: boolean;
  soundEnabled: boolean;
  allowTeamChanging: boolean;
  hideDevices: boolean;
}
