// Adding runtime type checking blows this file up by a lot. Sad times.
import { z } from "https://deno.land/x/zod@v3.23.8/mod.ts";

export type Transaction =
  | CreateInstanceTransaction
  | DeleteInstanceTransaction
  | JoinInstanceTransaction
  | CreateTeamTransaction
  | RenameTeamTransaction
  | SetTeamAltNameTransaction
  | RenameDeviceTransaction
  | JoinTeamTransaction
  | UpdateSettingsTransaction
  | SetBuzzersEnabledTransaction
  | DeleteTeamTransaction
  | SetPointsTransaction
  | ResetPointsTransaction
  | ChangeExclusionTransaction
  | BuzzTransaction
  | BuzzRandomTeamTransaction
  | DecrementCountdownTransaction
  | KeepAliveTransaction
  | ScheduledDeleteInstanceTransaction
  | ScheduledRearmTransaction;

export type SchedulableTransaction =
  | ScheduledDeleteInstanceTransaction
  | DecrementCountdownTransaction
  | ScheduledRearmTransaction;

const TimestampRt = z.object({ t: z.string() });
export type Timestamp = z.infer<typeof TimestampRt>;

// This file is generated by runtyping (https://github.com/johngeorgewright/runtyping).
// Manual changes might be lost - proceed with caution!
export const CreateInstanceTransactionRt = z.object({
  kind: z.literal("CreateInstance"),
  instanceId: z.string(),
  sessionId: z.string(),
  authKey: z.string(),
  timestamp: TimestampRt,
});

export type CreateInstanceTransaction = z.infer<
  typeof CreateInstanceTransactionRt
>;

export const DeleteInstanceTransactionRt = z.object({
  kind: z.literal("DeleteInstance"),
  sessionId: z.string(),
});

export type DeleteInstanceTransaction = z.infer<
  typeof DeleteInstanceTransactionRt
>;

export const JoinInstanceTransactionRt = z.object({
  kind: z.literal("JoinInstance"),
  instanceId: z.string(),
  sessionId: z.string(),
  deviceId: z.string(),
  authKey: z.string().optional(),
  name: z.string().optional(),
  timestamp: TimestampRt,
});

export type JoinInstanceTransaction = z.infer<typeof JoinInstanceTransactionRt>;

export const CreateTeamTransactionRt = z.object({
  kind: z.literal("CreateTeam"),
  sessionId: z.string(),
  name: z.string(),
  newTeamId: z.string(),
});

export type CreateTeamTransaction = z.infer<typeof CreateTeamTransactionRt>;

export const RenameTeamTransactionRt = z.object({
  kind: z.literal("RenameTeam"),
  sessionId: z.string(),
  name: z.string(),
  teamId: z.string().optional(),
});

export type RenameTeamTransaction = z.infer<typeof RenameTeamTransactionRt>;

export const SetTeamAltNameTransactionRt = z.object({
  kind: z.literal("SetTeamAltName"),
  sessionId: z.string(),
  name: z.string(),
  teamId: z.string().optional(),
});

export type SetTeamAltNameTransaction = z.infer<typeof SetTeamAltNameTransactionRt>;

export const RenameDeviceTransactionRt = z.object({
  kind: z.literal("RenameDevice"),
  sessionId: z.string(),
  deviceId: z.string().optional(),
  name: z.string(),
});

export type RenameDeviceTransaction = z.infer<typeof RenameDeviceTransactionRt>;

export const JoinTeamTransactionRt = z.object({
  kind: z.literal("JoinTeam"),
  sessionId: z.string(),
  deviceId: z.string().optional(),
  newTeamId: z.string(),
});

export type JoinTeamTransaction = z.infer<typeof JoinTeamTransactionRt>;

export const UpdateSettingsTransactionRt = z.object({
  kind: z.literal("UpdateSettings"),
  sessionId: z.string(),
  timeout: z.number(),
  rearmAfter: z.number(),
  autoReenable: z.boolean(),
  soundEnabled: z.boolean(),
  allowTeamChanging: z.boolean(),
  hideDevices: z.boolean(),
});

export type UpdateSettingsTransaction = z.infer<
  typeof UpdateSettingsTransactionRt
>;

export const SetBuzzersEnabledTransactionRt = z.object({
  kind: z.literal("SetBuzzersEnabled"),
  sessionId: z.string(),
  enabled: z.boolean(),
  timestamp: TimestampRt,
});

export type SetBuzzersEnabledTransaction = z.infer<
  typeof SetBuzzersEnabledTransactionRt
>;

export const ScheduledRearmTransactionRt = z.object({
  kind: z.literal("ScheduledRearm"),
  instanceId: z.string(),
  timestamp: TimestampRt,
  epoch: z.number(),
});

export type ScheduledRearmTransaction = z.infer<
  typeof ScheduledRearmTransactionRt
>;

export const DeleteTeamTransactionRt = z.object({
  kind: z.literal("DeleteTeam"),
  sessionId: z.string(),
  teamId: z.string().optional(),
});

export type DeleteTeamTransaction = z.infer<typeof DeleteTeamTransactionRt>;

export const SetPointsTransactionRt = z.object({
  kind: z.literal("SetPoints"),
  sessionId: z.string(),
  teamId: z.string(),
  points: z.number(),
});

export type SetPointsTransaction = z.infer<typeof SetPointsTransactionRt>;

export const ResetPointsTransactionRt = z.object({
  kind: z.literal("ResetPoints"),
  sessionId: z.string(),
});

export type ResetPointsTransaction = z.infer<typeof ResetPointsTransactionRt>;

export const ChangeExclusionTransactionRt = z.object({
  kind: z.literal("ChangeExclusion"),
  sessionId: z.string(),
  teamId: z.string(),
  excluded: z.boolean(),
});

export type ChangeExclusionTransaction = z.infer<
  typeof ChangeExclusionTransactionRt
>;

export const BuzzTransactionRt = z.object({
  kind: z.literal("Buzz"),
  sessionId: z.string(),
  teamId: z.string().optional(),
  timestamp: TimestampRt,
});

export type BuzzTransaction = z.infer<typeof BuzzTransactionRt>;

export const BuzzRandomTeamTransactionRt = z.object({
  kind: z.literal("BuzzRandomTeam"),
  sessionId: z.string(),
  timestamp: TimestampRt,
});

export type BuzzRandomTeamTransaction = z.infer<
  typeof BuzzRandomTeamTransactionRt
>;

export const DecrementCountdownTransactionRt = z.object({
  kind: z.literal("DecrementCountdown"),
  instanceId: z.string(),
  n: z.number(),
  timestamp: TimestampRt,
  epoch: z.number(),
});

export type DecrementCountdownTransaction = z.infer<
  typeof DecrementCountdownTransactionRt
>;

export const KeepAliveTransactionRt = z.object({
  kind: z.literal("KeepAlive"),
  sessionId: z.string(),
  timestamp: TimestampRt,
});

export type KeepAliveTransaction = z.infer<typeof KeepAliveTransactionRt>;

export const ScheduledDeleteInstanceTransactionRt = z.object({
  kind: z.literal("ScheduledDeleteInstance"),
  instanceId: z.string(),
  timestamp: TimestampRt,
});

export type ScheduledDeleteInstanceTransaction = z.infer<
  typeof ScheduledDeleteInstanceTransactionRt
>;

export const TransactionRt = z.discriminatedUnion("kind", [
  CreateInstanceTransactionRt,
  DeleteInstanceTransactionRt,
  JoinInstanceTransactionRt,
  CreateTeamTransactionRt,
  RenameTeamTransactionRt,
  RenameDeviceTransactionRt,
  JoinTeamTransactionRt,
  UpdateSettingsTransactionRt,
  SetBuzzersEnabledTransactionRt,
  DeleteTeamTransactionRt,
  SetPointsTransactionRt,
  ResetPointsTransactionRt,
  ChangeExclusionTransactionRt,
  BuzzTransactionRt,
  BuzzRandomTeamTransactionRt,
  DecrementCountdownTransactionRt,
  KeepAliveTransactionRt,
  ScheduledDeleteInstanceTransactionRt,
  ScheduledRearmTransactionRt,
]);
